/*

    Compute absolute solar system body positions
	
	         by John Walker
	    http://www.fourmilab.ch/
    
    For details, please see: http://www.fourmilab.ch/documents/canon_transits/
	
*/

#include <stdio.h>
#include <math.h>

#include "readout.h"
#include "absbody.h"
#include "linalg.h"
#include "astro.h"
#include "mathutil.h"
    
/*  Object names  */

const char *abs_objname[Moon + 1] = {
     "Librations", "Mercury", "Venus", "EMB",
     "Mars", "Jupiter", "Saturn", "Uranus",
     "Neptune", "Pluto", "Moon_Vec", "Sun",
     "Earth", "Moon"
};

/*  ABSBODY  --  Expand a state vector as returned by readout()
    	    	 to an absolute state vector containing the solar
		 system barycentric positions of the Earth and Moon.
		 These are computed by projecting the Moon_Vec from
		 the integration, which gives the solar system
		 barycentric coordinates of the Moon minus those
		 of the Earth from the Earth-Moon barycentre (EMB)
		 with magnitudes determined by the Earth/Moon mass
		 ratio.
*/

void absbody(abs_step *as, ss_step *s)
{
    int i;
    
    as->jd = s->jd;
    for (i = 0; i <= Sun; i++) {
    	as->obj[i] = s->obj[i];
    }
    
    VecAddS(-(1 / EARTH_MOON_MASS_RATIO), as->obj[Moon_Vec].pos, as->obj[EMB].pos,
    	    as->obj[Earth].pos);
    VecAddS(1 - (1 / EARTH_MOON_MASS_RATIO), as->obj[Moon_Vec].pos, as->obj[EMB].pos,
    	    as->obj[Moon].pos);
}


/*  READABS  --  Read next state vector record from file fi into
    	    	 abs_step s.  Returns 0 if the record was read
		 successfully or EOF if end of file was encountered
		 (including a truncated final record).  */

int readabs(FILE *fi, abs_step *as)
{
    ss_step s;
    int r = readout(fi, &s);
    
    if (r == 0) {
    	absbody(as, &s);
    }
    return r;
}
