
    #   Count local minima and maxima in bary_minmax.csv
    
    #	Input:	bary_minmax.csv     (Output from bary_minmax_csv.pl)
    #	Output: bary_local.csv	    (CSV database listing the number of minima and
    #	    	    	    	    maxima between each barycentre solar radius crossing)
    #	    	bary_local_hist.csv (CSV database containing a histogram of the number of
    #	    	    	    	    occurrence of each number of minima and maxima)
    
    #	by John Walker
    #	http://www.fourmilab.ch/
    
    #	For details, please see: http://www.fourmilab.ch/documents/canon_transits/
    
    use strict;
    use warnings;
    
    my $l;
    my $lastev = 'nothing';
    my ($lastjd, $lastcal);
    my $wiggles = 0;
    
    my @wistogram;

    open(FI, "<bary_minmax.csv") || die("Cannot open bary_minmax.csv");
    open(FO, ">bary_local.csv") || die("Cannot write bary_local.csv");
    while ($l = <FI>) {
	$l =~ s/\s+$//;
   	$l =~ m/^(\w+),([^,]+),([^,]+),(.*)$/ || die("Parse error on $l");
	my ($event, $G, $jd, $caldate) = ($1, $2, $3, $4);
	
	if ($event =~ m/^(enter|exit)/) {
	    if ($lastev ne 'nothing') {
	    	print(FO "$wiggles,$lastev,$lastjd,$lastcal,$event,$jd,$caldate\n");
		$wistogram[$wiggles]++;
	    }
	    $lastev = $event;
	    ($lastjd, $lastcal) = ($jd, $caldate);
	    $wiggles = 0;
	} else {
	    die("Invalid record in input file $l") if ($event !~ m/^(min|max)/);
	    $wiggles++;
    	}
    }
    close(FI);
    
    #	Output the histogram of the number of times we saw each number
    #	of minima and maxima.  Note that this number will always be odd
    #	(think about it).
    
    open(FO, ">bary_local_hist.csv") || die("Cannot write bary_local_hist.csv");
    for (my $i = 1; $i <= $#wistogram; $i += 2) {
    	print(FO "$i,$wistogram[$i]\n");
    }
    close(FO);
