
    #   Convert barycentre events in output from transits.c to
    #	barycentre event database.
    
    #	Input:	all_fine.txt.gz
    #	Output: bary_minmax.csv
    
    #	by John Walker
    #	http://www.fourmilab.ch/
    
    #	For details, please see: http://www.fourmilab.ch/documents/canon_transits/
    
    use strict;
    use warnings;
    require 'julian.pl';
    
    my $l;
    my (@elist, $nmax);
    
    open(FI, "zcat ../Integration_Results/all_fine.txt.gz |") ||
    	die("Cannot open pipe from ../Integration_Results/all_fine.txt.gz");
    open(FO, ">bary_minmax.csv") || die("Cannot write bary_minmax.csv");

    $nmax = -1;
    my $lastev = 'nothing';
    my ($Levent, $Ljd, $Lcivil, $LG);
    my $cevent;
    while ($l = <FI>) {
    	if ($l =~ m/^Barycentre (\w+) at ([\d\.\-]+)\s+(.*) \(G = ([\d\.\-]+)/) {
	    my ($event, $jd, $civil, $G) = ($1, $2, $3, $4);

	    if (($event eq 'exiting') || ($event eq 'entering')) {
	    	if ($LG == 1) {
		    $event =  ($lastev eq 'exiting') ? 'entering' : 'exiting';
		} else {
	    	    $event =  ($LG > 1) ? 'entering' : 'exiting';
		}
	    	if ($lastev ne 'nothing') {
		    die("Ugh: $jd $lastev -> $event") if ($event eq $lastev);
		}
		flushElist();
		$lastev = $event;
		$cevent = ($event eq 'exiting') ? "exit" : "enter";
	    	my $cd = JDtoCalendar($jd);
	    	print(FO "$cevent,$G,$jd,$cd\n");
    	    } else {
	    	die("G ($G) inconsistent with Lastev ($lastev) at $jd") if (($lastev eq 'exiting') && ($G < 1));
	    	die("G ($G) inconsistent with Lastev ($lastev) at $jd") if (($lastev eq 'entering') && ($G > 1));
		$cevent = ($event eq 'minimum') ? "min" : "max";
	    	my $cd = JDtoCalendar($jd);
		push(@elist, "$cevent,$G,$jd,$cd");
    	    }
	    ($Levent, $Ljd, $Lcivil, $LG) = ($event, $jd, $civil, $G);
    	}
    }
    flushElist();
    close(FI);
    close(FO);
    
    sub flushElist {
    	my $l;
	
	my $gext = ($lastev eq 'exiting') ? -1 : 2;
	for (my $i = 0; $i <= $#elist; $i++) {
	    $elist[$i] =~ m/\w,([^,]+),/ || die("Parse error 1 in flushElist on $elist[$i]");
	    my $g = $1;
	    if ($lastev eq 'exiting') {
	    	if ($g > $gext) {
		    $gext = $g;
		}
	    } else {
	    	if ($g < $gext) {
		    $gext = $g;
		}
	    }
    	}
	
	while ($l = shift(@elist)) {
	    $l =~ m/\w,([^,]+),/ || die("Parse error 2 in flushElist on $l");
	    my $g = $1;
	    if ($g == $gext) {
	    	$l =~ s/,/G,/;
	    }
	    print(FO "$l\n");
	}
    }
